import pytest
from kubernetes import client, config
from kubernetes.client.rest import ApiException
import os


def get_release(name, namespace, api):
    print('Testing %s in namespace %s ...' % (name, namespace), end='')
    try:
        release = api.get_namespaced_custom_object(
            group="helm.fluxcd.io",
            version="v1",
            plural="helmreleases",
            name=name,
            namespace=namespace
        )
        release_status = release['status']['releaseStatus']
        print(release_status)
    except ApiException as e:
        if e.status == 404:
            release_status = 'not found'
        else:
            raise
        print("**** NOT DEPLOYED, status: %s *****" % release_status)

    return(release_status)


@pytest.mark.helmreleases
def test_helmreleases(host):
    """Checks if all desired HelmReleases installed by weave flux are in
    DEPLOYED state.
    """
    expected_releases = {
        'oas': ['cert-manager', 'ingress', 'local-storage', 'monitoring'],
        'oas-apps': ['nextcloud']
    }

    cluster_dir = os.environ.get("CLUSTER_DIR")
    kubeconfig = os.path.join(cluster_dir, 'secrets',
                              'kube_config_cluster.yml')
    config.load_kube_config(config_file=kubeconfig)
    customObjects = client.CustomObjectsApi()

    failed = 0
    print('\n')
    for namespace in expected_releases:
        for app in expected_releases[namespace]:
            app_status = get_release(app, namespace, customObjects)
            if app_status != 'DEPLOYED':
                failed += 1
    assert failed == 0, "Error: %s apps NOT DEPLOYED !" % failed


if __name__ == "__main__":
    test_helmreleases()
